<?php
namespace Mince;

use Index\Performance\Timings;
use Index\Data\DbType;
use Index\Http\HttpFx;
use Mince\HTML;

require_once __DIR__ . '/../mince.php';

$router = new HttpFx;

$loginUrl = $config['login_url'];

$router->setDefaultErrorHandler(function($response, $request, $code, $text) use ($loginUrl, $userInfo) {
    $body = HTML::getHeader($userInfo, $loginUrl, sprintf('%03d %s', $code, $text));
    $body .= '<div class="error">';
    $body .= sprintf('    <h2>HTTP %03d</h2>', $code);
    $body .= sprintf('    <p>%s</p>', $text);
    $body .= '</div>';
    $body .= HTML::getFooter();
    $response->setContent($body);
});

$router->use('/', function($response) {
    $response->setPoweredBy('Mince');
});

$router->get('/index.php', function($response) {
    $response->redirect('/', true);
});

$router->get('/', function($response, $request) use ($db, $remote, $loginUrl, $userInfo, $sVerification) {
    $name = (string)$request->getParam('name');
    $error = (string)$request->getParam('error');

    if(!empty($error) && ctype_lower($error)) {
        $errors = [
            'request' => ['Invalid request type.', 'Try to reload the page and try again.'],
            'verify' => ['Request verification failed.', 'Try to reload the page and try again.'],
            'itainthappenin' => ['Haha', 'No'],
            'short' => ['Invalid username', 'The provided name is too short.'],
            'long' => ['Invalid username', 'The provided name is too long.'],
            'invalid' => ['Invalid username', 'The provided name contains invalid characters.'],
            'conflict' => ['Username conflict', 'This username is already whitelisted with someone, contact flashwave if this is unexpected.'],
            'connect' => ['Failed to connect to the server', 'The server is probably offline, pope flashwave if this is not expected.'],
            'not-listed' => ['You have not been whitelisted yet', 'Add yourself to the whitelist before trying to remove yourself from it.'],
        ];

        if(array_key_exists($error, $errors)) {
            $mErrorTitle = $errors[$error][0];
            $mErrorComment = $errors[$error][1];
        } else {
            $mErrorTitle = 'Unexpected response from server';
            $mErrorComment = $error;
        }
    }

    $body = HTML::getHeader($userInfo, $loginUrl);

    if(!empty($mErrorTitle)) {
        $body .= '<div class="error">';
        $body .= sprintf('    <h2>%s</h2>', $mErrorTitle);
        $body .= sprintf('    <p>%s</p>', $mErrorComment ?? 'No further details provided.');
        $body .= '</div>';
    }

    if($userInfo->success) {
        if($userInfo->mc_whitelisted < 1) {
            $body .= '<div class="whitelist">';
            $body .= '    <h2>Add to Whitelist</h2>';
            $body .= '    <p>This will give you access to the server.</p>';
            $body .= '    <form method="post" action="/whitelist/add">';
            $body .= sprintf('        <input type="hidden" name="boob" value="%s">', $sVerification);
            $body .= '        <label>';
            $body .= '            <div class="label-header">Username</div>';
            $body .= sprintf('            <div class="label-input"><input type="text" name="name" value="%s"></div>', htmlspecialchars($name));
            $body .= '        </label>';
            $body .= '        <input type="submit" value="Add me to the Whitelist">';
            $body .= '    </form>';
            $body .= '</div>';
        }

        $body .= '<div class="details">';
        $body .= '    <h2>Server address</h2>';
        $body .= '    <p style="color: red;">Down currently because I don\'t have the resources to run multiple games at once yet.</p>';
        //$body .= '    <pre><code>mc.flashii.net</code></pre>';
        //$body .= '    <p>An SRV record is used, so no port needs to be specified.</p>';
        $body .= '    <p>We recommend you use <a href="https://multimc.org/" target="_blank" rel="noopener">MultiMC</a>/<a href="https://polymc.org/" target="_blank" rel="noopener">PolyMC</a> for easy version and mod management and <a href="https://adoptium.net/">Adoptium Java 17</a> to actually be able to launch the game. Other launchers work completely fine though.</p>';
        $body .= '</div>';

        $body .= '<div class="bedrock">';
        $body .= '    <h2>Bedrock versions</h2>';
        $body .= '    <p>Through the black magic bestowed upon us by <a href="https://geysermc.org/" target="_blank" rel="noopener">GeyserMC</a> it\'s possible to play on the server through any of the updated Bedrock versions of Minecraft.</p>';
        $body .= '    <p>This should allow you to play on the server from a phone, a tablet or a console, provided you also have an account for the original version of the game.</p>';
        $body .= '    <p>You will need to link your Minecraft and Bedrock accounts, you can do this by connecting to <code>link.geysermc.org</code> in both versions of the game and following the on-screen instructions.</p>';
        $body .= '    <p>Unfortunately, the Bedrock versions of the game don\'t support SRV records, so you\'ll have to join with a different address:</p>';
        //$body .= '    <pre><code>mcb.flashii.net:19132</code></pre>';
        $body .= '    <p style="color: red;">Also down, lol!</p>';
        $body .= '</div>';
        
        if($userInfo->mc_whitelisted > 0) {
            $body .= '<div class="unwhitelist">';
            $body .= '    <h2>Remove from Whitelist</h2>';
            $body .= '    <p>This will revoke your access to the server.</p>';
            $body .= sprintf('    <p>You are currently whitelisted as <b>%s</b> on <b>%s</b>.</p>', $userInfo->mc_username, date('Y-m-d H:i:s T', $userInfo->mc_whitelisted));
            $body .= '    <form method="post" action="/whitelist/remove">';
            $body .= sprintf('        <input type="hidden" name="boob" value="%s">', $sVerification);
            $body .= '        <input type="submit" value="Remove me from the whitelist">';
            $body .= '    </form>';
            $body .= '</div>';
        }

        $body .= '<div class="bedrock">';
        $body .= '    <h2>All Of Fabric 5 (1.3.0)</h2>';
        $body .= '    <pre><code>aof.flashii.net</code></pre>';
        $body .= '    <p>An SRV record is used, so no port needs to be specified.</p>';
        $body .= '    <p>There\'s a pack for <a href="https://multimc.org/" target="_blank" rel="noopener">MultiMC</a>/<a href="https://polymc.org/" target="_blank" rel="noopener">PolyMC</a> somewhere and you\'ll probably need <a href="https://adoptium.net/">Adoptium Java 17</a> to actually be able to launch the game. Other launchers work completely fine though, probably.</p>';
        $body .= '    <p>This server doesn\'t currently use a whitelist, so you don\'t have to worry about that.</p>';
        $body .= '</div>';
    } else {
        $body .= '<div class="loggedout">';
        $body .= '    <h2>You must be logged in to use this website!</h2>';
        $body .= '    <p>This website allows you to whitelist yourself on our Minecraft server, for which you need to be logged in.</p>';
        $body .= '    <p>So it doesn\'t make sense to display the details either.</p>';
        $body .= '</div>';
    }

    $body .= '<div class="rules">';
    $body .= '    <h2>Rules</h2>';
    $body .= '    <p>1. Observe <a href="https://flashii.net/info/rules">Global Rules</a>.</p>';
    $body .= '    <p>2. Don\'t be an asshole.</p>';
    $body .= '    <p>3. Don\'t flood.</p>';
    $body .= '</div>';

    $body .= HTML::getFooter();

    return $body;
});

$router->use('/whitelist', function($response, $request) use ($sVerification) {
    if(!$request->isFormContent()) {
        $response->redirect('/?error=request');
        return true;
    }

    $body = $request->getContent();

    if(!$body->hasParam('boob') || !hash_equals($sVerification, (string)$body->getParam('boob'))) {
        $response->redirect('/?error=verify');
        return true;
    }
});

$router->post('/whitelist/add', function($response, $request) use ($db, $userInfo) {
    if($userInfo->user_id == 45) {
        $response->redirect('/?error=itainthappenin');
        return true;
    }

    $body = $request->getContent();
    $name = (string)$body->getParam('name');
    $resp = (new Whitelist($db))->add($userInfo, $name);

    if($resp === '')
        $response->redirect('/');
    else {
        if($resp === 'invalid')
            $name = '';
        $response->redirect("/?error={$resp}&name={$name}");
    }
});

$router->post('/whitelist/remove', function($response) use ($db, $userInfo) {
    $resp = (new Whitelist($db))->remove($userInfo);

    if($resp === '')
        $response->redirect('/');
    else
        $response->redirect("/?error={$resp}");
});

$router->get('/errors/:code', function($res, $req, $code) {
    $code = intval($code);
    if($code < 100 || $code >= 600)
        $code = 400;
    return $code;
});

$router->dispatch();
